@@ -55,7 +55,7 @@
 
 #if defined(LIBC_SCCS) && !defined(lint)
 static char sccsid[] = "@(#)res_debug.c	8.1 (Berkeley) 6/4/93";
-static char rcsid[] = "res_debug.c,v 1.2 1995/05/06 14:12:41 hjl Exp";
+static char rcsid[] = "$Id: res_debug.c,v 1.6 1996/10/04 22:52:52 hjl Exp $";
 #endif /* LIBC_SCCS and not lint */
 
 #include <sys/param.h>
@@ -74,6 +74,51 @@ extern const char *_res_opcodes[];
 extern const char *_res_resultcodes[];
 
 #ifdef __ELF__
+#ifdef __GNUC__
+        char *__p_time         ( u_int32_t );
+const   char *__p_option       ( u_long );
+const   char *__p_class        ( int );
+const   char *__p_type         ( int );
+const u_char *__p_rr           ( const u_char *, const u_char *, FILE * );
+const u_char *__p_fqname       ( const u_char *, const u_char *, FILE * );
+const u_char *__p_cdname       ( const u_char *, const u_char *, FILE * );
+const u_char *__p_cdnname    ( const u_char *, const u_char *, int, FILE * );
+void         __fp_query        ( const u_char *, FILE * );
+void         __fp_nquery       ( const u_char *, int, FILE * );
+void         __fp_resstat      ( struct __res_state *, FILE * );
+void         __p_query         ( const u_char * );
+static const u_char *do_rrset  ( const u_char *, int, const u_char *,
+                     int, int, FILE *, const char * );
+static const char   *deproto   ( int );
+static const char   *dewks     ( int );
+/* 
+ * This is sure a grotesque way to avoid ANSI standards. vch
+ */
+        char *p_time         ( u_int32_t )
+    __attribute__ ((weak, alias ("__p_time")));
+const   char *p_option       ( u_long )
+    __attribute__ ((weak, alias ("__p_option")));
+const   char *p_class        ( int )
+    __attribute__ ((weak, alias ("__p_class")));
+const   char *p_type         ( int )
+    __attribute__ ((weak, alias ("__p_type")));
+const u_char *p_rr           ( const u_char *, const u_char *, FILE * )
+    __attribute__ ((weak, alias ("__p_rr")));
+const u_char *p_fqname       ( const u_char *, const u_char *, FILE * )
+    __attribute__ ((weak, alias ("__p_fqname")));
+const u_char *p_cdname       ( const u_char *, const u_char *, FILE * )
+    __attribute__ ((weak, alias ("__p_cdname")));
+const u_char *p_cdnname    ( const u_char *, const u_char *, int, FILE * )
+    __attribute__ ((weak, alias ("__p_cdnname")));
+void         fp_query        ( const u_char *, FILE * )
+    __attribute__ ((weak, alias ("__fp_query")));
+void         fp_nquery       ( const u_char *, int, FILE * )
+    __attribute__ ((weak, alias ("__fp_nquery")));
+void         fp_resstat      ( struct __res_state *, FILE * )
+    __attribute__ ((weak, alias ("__fp_resstat")));
+void         p_query         ( const u_char * )
+    __attribute__ ((weak, alias ("__p_query")));
+#else
 #pragma weak fp_query = __fp_query
 #pragma weak fp_nquery = __fp_nquery
 #pragma weak fp_resstat = __fp_resstat
@@ -86,12 +131,12 @@ extern const char *_res_resultcodes[];
 #pragma weak p_rr = __p_rr
 #pragma weak p_time = __p_time
 #pragma weak p_type = __p_type
-#endif
+#endif   /* __GNUC__ */
+#endif   /* __ELF__ */
 
 /* XXX: we should use getservbyport() instead. */
 static const char *
-dewks(wks)
-	int wks;
+dewks( int wks )
 {
 	static char nbuf[20];
 
@@ -148,8 +193,7 @@ dewks(wks)
 
 /* XXX: we should use getprotobynumber() instead. */
 static const char *
-deproto(protonum)
-	int protonum;
+deproto( int protonum )
 {
 	static char nbuf[20];
 
@@ -171,11 +215,8 @@ deproto(protonum)
 }
 
 static const u_char *
-do_rrset(msg, len, cp, cnt, pflag, file, hs)
-	int cnt, pflag, len;
-	const u_char *cp, *msg;
-	const char *hs;
-	FILE *file;
+do_rrset( const u_char *msg, int len, const u_char *cp, int cnt,
+			int pflag, FILE *file, const char *hs)
 {
 	int n;
 	int sflag;
@@ -184,7 +225,7 @@ do_rrset(msg, len, cp, cnt, pflag, file, hs)
 	 * Print answer records.
 	 */
 	sflag = (_res.pfcode & pflag);
-	if (n = ntohs(cnt)) {
+	if ((n = ntohs(cnt))) {
 		if ((!_res.pfcode) ||
 		    ((sflag) && (_res.pfcode & RES_PRF_HEAD1)))
 			fprintf(file, hs);
@@ -212,8 +253,7 @@ do_rrset(msg, len, cp, cnt, pflag, file, hs)
 }
 
 void
-__p_query(msg)
-	const u_char *msg;
+__p_query( const u_char *msg )
 {
 	__fp_query(msg, stdout);
 }
@@ -223,10 +263,9 @@ __p_query(msg)
  * Print the current options.
  * This is intended to be primarily a debugging routine.
  */
+
 void
-__fp_resstat(statp, file)
-	struct __res_state *statp;
-	FILE *file;
+__fp_resstat( struct __res_state *statp, FILE *file )
 {
 	register u_long mask;
 
@@ -243,11 +282,9 @@ __fp_resstat(statp, file)
  * Print the contents of a query.
  * This is intended to be primarily a debugging routine.
  */
+
 void
-__fp_nquery(msg, len, file)
-	const u_char *msg;
-	int len;
-	FILE *file;
+__fp_nquery( const u_char *msg, int len, FILE *file )
 {
 	register const u_char *cp, *endMark;
 	register const HEADER *hp;
@@ -300,7 +337,7 @@ __fp_nquery(msg, len, file)
 	/*
 	 * Print question records.
 	 */
-	if (n = ntohs(hp->qdcount)) {
+	if ((n = ntohs(hp->qdcount))) {
 		if ((!_res.pfcode) || (_res.pfcode & RES_PRF_QUES))
 			fprintf(file, ";; QUESTIONS:\n");
 		while (--n >= 0) {
@@ -366,18 +403,13 @@ __fp_nquery(msg, len, file)
 }
 
 void
-__fp_query(msg, file)
-	const u_char *msg;
-	FILE *file;
+__fp_query( const u_char *msg, FILE *file )
 {
 	fp_nquery(msg, PACKETSZ, file);
 }
 
 const u_char *
-__p_cdnname(cp, msg, len, file)
-	const u_char *cp, *msg;
-	int len;
-	FILE *file;
+__p_cdnname( const u_char *cp, const u_char *msg, int len, FILE *file )
 {
 	char name[MAXDNAME];
 	int n;
@@ -392,9 +424,7 @@ __p_cdnname(cp, msg, len, file)
 }
 
 const u_char *
-__p_cdname(cp, msg, file)
-	const u_char *cp, *msg;
-	FILE *file;
+__p_cdname( const u_char *cp, const u_char *msg, FILE *file )
 {
 	return (p_cdnname(cp, msg, PACKETSZ, file));
 }
@@ -403,9 +433,7 @@ __p_cdname(cp, msg, file)
  */
 
 const u_char *
-__p_fqname(cp, msg, file)
-	const u_char *cp, *msg;
-	FILE *file;
+__p_fqname( const u_char *cp, const u_char *msg, FILE *file )
 {
 	char name[MAXDNAME];
 	int n;
@@ -426,9 +454,7 @@ __p_fqname(cp, msg, file)
  * Print resource record fields in human readable form.
  */
 const u_char *
-__p_rr(cp, msg, file)
-	const u_char *cp, *msg;
-	FILE *file;
+__p_rr( const u_char *cp, const u_char *msg, FILE *file )
 {
 	int type, class, dlen, n, c;
 	struct in_addr inaddr;
@@ -501,7 +527,7 @@ __p_rr(cp, msg, file)
 	case T_HINFO:
 	case T_ISDN:
 		cp2 = cp + dlen;
-		if (n = *cp++) {
+		if ((n = *cp++)) {
 			fprintf(file, "\t%.*s", n, cp);
 			cp += n;
 		}
@@ -560,7 +586,7 @@ __p_rr(cp, msg, file)
 		(void) fputs("\t\"", file);
 		cp2 = cp1 + dlen;
 		while (cp < cp2) {
-			if (n = (unsigned char) *cp++) {
+			if ((n = (unsigned char) *cp++)) {
 				for (c = n; c > 0 && cp < cp2; c--)
 					if ((*cp == '\n') || (*cp == '"')) {
 					    (void) putc('\\', file);
@@ -668,8 +694,7 @@ __p_rr(cp, msg, file)
  * Return a string for the type
  */
 const char *
-__p_type(type)
-	int type;
+__p_type( int type )
 {
 	static char nbuf[20];
 
@@ -719,8 +744,7 @@ __p_type(type)
  * Return a mnemonic for class
  */
 const char *
-__p_class(class)
-	int class;
+__p_class( int class )
 {
 	static char nbuf[20];
 
@@ -736,8 +760,7 @@ __p_class(class)
  * Return a mnemonic for an option
  */
 const char *
-__p_option(option)
-	u_long option;
+__p_option( u_long option )
 {
 	static char nbuf[40];
 
@@ -763,8 +786,7 @@ __p_option(option)
  * Return a mnemonic for a time to live
  */
 char *
-__p_time(value)
-	u_int32_t value;
+__p_time( u_int32_t value )
 {
 	static char nbuf[120];
 	int secs, mins, hours, days;
