@@ -1,4 +1,4 @@
-/* 
+/*
 Copyright (C) 1993, 1995 Free Software Foundation
 
 This file is part of the GNU IO Library.  This library is free
@@ -24,7 +24,9 @@ the executable file might be covered by the GNU General Public License. */
 
 /*  written by Per Bothner (bothner@cygnus.com) */
 
-#define _POSIX_SOURCE
+#ifndef _POSIX_SOURCE
+# define _POSIX_SOURCE
+#endif
 #include "libioP.h"
 #include <fcntl.h>
 #include <sys/types.h>
@@ -35,11 +37,16 @@ the executable file might be covered by the GNU General Public License. */
 extern int errno;
 #endif
 
+
+#ifdef _LIBC
+# define open(Name, Flags, Prot) __open ((Name), (Flags), (Prot))
+#endif
+
 /* An fstream can be in at most one of put mode, get mode, or putback mode.
    Putback mode is a variant of get mode.
 
    In a filebuf, there is only one current position, instead of two
-   separate get and put pointers.  In get mode, the current posistion
+   separate get and put pointers.  In get mode, the current position
    is that of gptr(); in put mode that of pptr().
 
    The position in the buffer that corresponds to the position
@@ -73,14 +80,14 @@ extern int errno;
    (The pointers save_gptr() and save_egptr() are the values
    of gptr() and egptr() at the time putback mode was entered.)
    The OS position corresponds to that of save_egptr().
-   
+
    LINE BUFFERED OUTPUT:
    During line buffered output, _IO_write_base==base() && epptr()==base().
    However, ptr() may be anywhere between base() and ebuf().
    This forces a call to filebuf::overflow(int C) on every put.
    If there is more space in the buffer, and C is not a '\n',
    then C is inserted, and pptr() incremented.
-   
+
    UNBUFFERED STREAMS:
    If a filebuf is unbuffered(), the _shortbuf[1] is used as the buffer.
 */
@@ -100,9 +107,7 @@ DEFUN(_IO_file_init, (fp),
   fp->_IO_file_flags |= CLOSED_FILEBUF_FLAGS;
 
   _IO_link_in(fp);
-#if 0
   fp->_fileno = -1;
-#endif
 }
 
 int
@@ -133,8 +138,8 @@ DEFUN(_IO_file_close_it, (fp),
 }
 
 void
-DEFUN(_IO_file_finish, (fp),
-      register _IO_FILE* fp)
+DEFUN(_IO_file_finish, (fp, dummy),
+      register _IO_FILE* fp AND int dummy)
 {
   if (_IO_file_is_open(fp))
     {
@@ -142,7 +147,7 @@ DEFUN(_IO_file_finish, (fp),
       if (!(fp->_flags & _IO_DELETE_DONT_CLOSE))
 	_IO_SYSCLOSE (fp);
     }
-  _IO_default_finish(fp);
+  _IO_default_finish(fp, 0);
 }
 
 _IO_FILE *
@@ -184,7 +189,7 @@ DEFUN(_IO_file_fopen, (fp, filename, mode),
   _IO_mask_flags(fp, read_write,_IO_NO_READS+_IO_NO_WRITES+_IO_IS_APPENDING);
   if (read_write & _IO_IS_APPENDING)
     if (_IO_SEEKOFF (fp, (_IO_off_t)0, _IO_seek_end, _IOS_INPUT|_IOS_OUTPUT)
-      == _IO_pos_BAD && errno != ESPIPE)
+	== _IO_pos_BAD && errno != ESPIPE)
       return NULL;
   _IO_link_in(fp);
   return fp;
@@ -223,11 +228,11 @@ DEFUN(_IO_file_setbuf, (fp, p, len),
 }
 
 /* Write TO_DO bytes from DATA to FP.
-   Then mark FP has having empty buffers. */
+   Then mark FP as having empty buffers. */
 
 int
 DEFUN(_IO_do_write, (fp, data, to_do),
-  register _IO_FILE *fp AND const char* data AND _IO_size_t to_do)
+      register _IO_FILE *fp AND const char* data AND _IO_size_t to_do)
 {
   _IO_size_t count;
   if (to_do == 0)
@@ -240,7 +245,7 @@ DEFUN(_IO_do_write, (fp, data, to_do),
        unpredictable. */
     fp->_offset = _IO_pos_BAD;
   else if (fp->_IO_read_end != fp->_IO_write_base)
-    { 
+    {
       _IO_pos_t new_pos
 	= _IO_SYSSEEK(fp, fp->_IO_write_base - fp->_IO_read_end, 1);
       if (new_pos == _IO_pos_BAD)
@@ -269,7 +274,10 @@ DEFUN(_IO_file_underflow, (fp),
 #endif
 
   if (fp->_flags & _IO_NO_READS)
-    return EOF;
+    {
+      errno = EBADF;
+      return EOF;
+    }
   if (fp->_IO_read_ptr < fp->_IO_read_end)
     return *(unsigned char*)fp->_IO_read_ptr;
 
@@ -287,28 +295,10 @@ DEFUN(_IO_file_underflow, (fp),
 		       fp->_IO_buf_end - fp->_IO_buf_base);
   if (count <= 0)
     {
-      /* 
-       * I think EAGAIN and EWOULDBLOCK should be treated as
-       * EOF. H.J.
-       */
-#ifdef EAGAIN
-#if defined(EWOULDBLOCK) && EWOULDBLOCK != EAGAIN
-      if (count == 0 || errno == EAGAIN
-	|| errno == EWOULDBLOCK)
-#else
-      if (count == 0 || errno == EAGAIN)
-#endif
-#else
-#ifdef EWOULDBLOCK
-      if (count == 0 || errno == EWOULDBLOCK)
-#else
       if (count == 0)
-#endif
-#endif
 	fp->_flags |= _IO_EOF_SEEN;
       else
-	fp->_flags |= _IO_ERR_SEEN;
-      count = 0;
+	fp->_flags |= _IO_ERR_SEEN, count = 0;
   }
   fp->_IO_read_base = fp->_IO_read_ptr = fp->_IO_buf_base;
   fp->_IO_read_end = fp->_IO_buf_base + count;
@@ -326,10 +316,11 @@ DEFUN(_IO_file_overflow, (f, ch),
       register _IO_FILE* f AND int ch)
 {
   if (f->_flags & _IO_NO_WRITES) /* SET ERROR */
-  {
-    f->_flags |= _IO_ERR_SEEN;
-    return EOF;
-  }
+    {
+      f->_flags |= _IO_ERR_SEEN;
+      errno = EBADF;
+      return EOF;
+    }
   /* If currently reading or no buffer allocated. */
   if ((f->_flags & _IO_CURRENTLY_PUTTING) == 0)
     {
@@ -378,7 +369,7 @@ DEFUN(_IO_file_sync, (fp),
   /*    char* ptr = cur_ptr(); */
   if (fp->_IO_write_ptr > fp->_IO_write_base)
     if (_IO_do_flush(fp)) return EOF;
-  delta = fp->_IO_read_ptr - fp->_IO_read_end; 
+  delta = fp->_IO_read_ptr - fp->_IO_read_end;
   if (delta != 0)
     {
 #ifdef TODO
@@ -408,6 +399,10 @@ DEFUN(_IO_file_seekoff, (fp, offset, dir, mode),
   _IO_pos_t result;
   _IO_off_t delta, new_offset;
   long count;
+  /* POSIX.1 8.2.3.7 says that after a call the fflush() the file
+     offset of the underlying file must be exact.  */
+  int must_be_exact = (fp->_IO_read_base == fp->_IO_read_end
+		       && fp->_IO_write_base == fp->_IO_write_ptr);
 
   if (mode == 0)
     dir = _IO_seek_cur, offset = 0; /* Don't move any pointers. */
@@ -524,7 +519,8 @@ DEFUN(_IO_file_seekoff, (fp, offset, dir, mode),
   else
     {
       count = _IO_SYSREAD (fp, fp->_IO_buf_base,
-			   fp->_IO_buf_end - fp->_IO_buf_base);
+			   (must_be_exact
+			    ? delta : fp->_IO_buf_end - fp->_IO_buf_base));
       if (count < delta)
 	{
 	  /* We weren't allowed to read, but try to seek the remainder. */
@@ -557,7 +553,11 @@ DEFUN(_IO_file_read, (fp, buf, size),
   for (;;)
     {
       _IO_ssize_t count = _IO_read(fp->_fileno, buf, size);
-#ifdef EINTR
+#if 0 && defined EINTR
+      /* We must not do this optimization since POSIX.1 explicitly
+	 requests that the stream operations must return with the
+	 error EINTR if this happens.  There must be the possibility
+	 that stream operations time out.  --drepper  */
       if (count == -1 && errno == EINTR)
 	continue;
 #endif
@@ -596,7 +596,11 @@ DEFUN(_IO_file_write, (f, data, n),
       _IO_ssize_t count = _IO_write(f->_fileno, data, to_do);
       if (count == EOF)
 	{
-#ifdef EINTR
+#if 0 && defined EINTR
+	  /* We must not do this optimization since POSIX.1 explicitly
+	     requests that the stream operations must return with the
+	     error EINTR if this happens.  There must be the
+	     possibility that stream operations time out.  --drepper  */
 	  if (errno == EINTR)
 	    continue;
 	  else
@@ -680,7 +684,7 @@ DEFUN(_IO_file_xsputn, (f, data, n),
       if (_IO_do_write(f, s, count) == EOF)
 	return n - to_do;
       to_do = dont_write;
-      
+
       /* Now write out the remainder.  Normally, this will fit in the
 	 buffer, but it's somewhat messier for line-buffered files,
 	 so we let _IO_default_xsputn handle the general case. */
@@ -745,7 +749,7 @@ DEFUN(_IO_file_xsgetn, (fp, data, n),
 	       else
 		 fp->_flags |= _IO_ERR_SEEN, count = 0;
 	     }
-	  
+
 	  s += count;
 	  more -= count;
 	}
