@@ -30,13 +30,13 @@ CONST char *CONST _sys_siglist[] =
     "Stopped",
     "Stopped (tty input)",
     "Stopped (tty output)",
-    "Possible I/O",
+    "Urgent condition",
     "CPU time limit exceeded",
     "File size limit exceeded",
     "Virtual time alarm",
     "Profile signal",
     "Window size changed",
-    "File lock lost",
+    "Possible I/O",
     "Power failure",
     "Unused signal",
     NULL
