@@ -244,8 +244,17 @@ DEFUN (strptime, (buf, format, tm),
 	  break;
 	case 'Y':
 	  /* Match year including century number.  */
-	  get_number (0, INT_MAX);
-	  tm->tm_year = val - (val >= 2000 ? 2000 : 1900);
+	  /* Use 9999 instead of INT_MAX. That is best we can do.
+	   * Let others worry about a year 9999 problem. H.J. */
+	  if (sizeof (time_t) > 4)
+	  {
+	    get_number (0, 9999);
+	  }
+	  else
+	  {
+	    get_number (0, 2038);
+	  }
+	  tm->tm_year = val - 1900;
 	  break;
 	case 'Z':
 	  /* XXX How to handle this?  */
